@@ -395,9 +395,11 @@ inline T D3DX12AlignAtLeast(T uValue, T uAlign)
 
 inline const CD3DX12_RESOURCE_DESC1* D3DX12ConditionallyExpandAPIDesc(
     D3D12_RESOURCE_DESC1& LclDesc,
-    const D3D12_RESOURCE_DESC1* pDesc)
+    const D3D12_RESOURCE_DESC1* pDesc,
+    const bool tightAlignmentSupported = false,
+    const bool alignAsCommitted = false)
 {
-    return D3DX12ConditionallyExpandAPIDesc(static_cast<CD3DX12_RESOURCE_DESC1&>(LclDesc), static_cast<const CD3DX12_RESOURCE_DESC1*>(pDesc));
+    return D3DX12ConditionallyExpandAPIDesc(static_cast<CD3DX12_RESOURCE_DESC1&>(LclDesc), static_cast<const CD3DX12_RESOURCE_DESC1*>(pDesc), tightAlignmentSupported, alignAsCommitted);
 }
 
 #if defined(D3D12_SDK_VERSION) && (D3D12_SDK_VERSION >= 606)
@@ -429,6 +431,12 @@ inline bool D3DX12GetCopyableFootprints(
     // Check if its a valid format
     D3DX12_ASSERT(D3D12_PROPERTY_LAYOUT_FORMAT_TABLE::FormatExists(Format));
 
+    // D3DX12GetCopyableFootprints does not support buffers with width larger than UINT_MAX.
+    if (ResourceDesc.Dimension == D3D12_RESOURCE_DIMENSION_BUFFER && ResourceDesc.Width >= UINT_MAX)
+    {
+        return false;
+    }
+
     const UINT WidthAlignment = D3D12_PROPERTY_LAYOUT_FORMAT_TABLE::GetWidthAlignment( Format );
     const UINT HeightAlignment = D3D12_PROPERTY_LAYOUT_FORMAT_TABLE::GetHeightAlignment( Format );
     const UINT16 DepthAlignment = UINT16( D3D12_PROPERTY_LAYOUT_FORMAT_TABLE::GetDepthAlignment( Format ) );
@@ -460,7 +468,7 @@ inline bool D3DX12GetCopyableFootprints(
         UINT32 MinPlanePitchWidth, PlaneWidth, PlaneHeight;
         D3D12_PROPERTY_LAYOUT_FORMAT_TABLE::GetPlaneSubsampledSizeAndFormatForCopyableLayout(PlaneSlice, Format, (UINT)Width, Height, /*_Out_*/ PlaneFormat, /*_Out_*/ MinPlanePitchWidth, /* _Out_ */ PlaneWidth, /*_Out_*/ PlaneHeight);
 
-        D3D12_SUBRESOURCE_FOOTPRINT LocalPlacement;
+        D3D12_SUBRESOURCE_FOOTPRINT LocalPlacement = {};
         auto& Placement = pLayouts ? pLayouts[uSubRes].Footprint : LocalPlacement;
         Placement.Format = PlaneFormat;
         Placement.Width = PlaneWidth;
@@ -600,3 +608,5 @@ inline bool D3DX12GetCopyableFootprints(
 }
 
 #endif // D3D12_SDK_VERSION >= 606
+
+
